# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: 2021 Taneli Hukkinen
# Licensed to PSF under a Contributor Agreement.

# Type annotations
# ParseFloat = Callable[[str], Any]
# Key = Tuple[str, ...]
# Pos = int
